#! /bin/env/python3
#"Standar Library"
import sys
from collections import Counter, OrderedDict


#"Dependencias"
from sklearn.cluster import DBSCAN
from sklearn.cluster import AgglomerativeClustering
from sklearn import metrics
from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import axes3d
import numpy as np

import plot_table_comparing_classifications as plott

def read_curly(archivo, categoria):
    """
        Lee el archivo, devuelve un diccionario con la columna (categoria) y los ejes de cada muestra, siendo la key la etiqueta

        input:
        -----------
        archivo: el archivo .curly
        categoría: la columna que contiene los datos a clasificar
    """
    sample_info = OrderedDict()
    with open(archivo) as t_file:
        t_file.readline()
        for line in t_file:
            sample_line = line.strip("\n").split("\t")
            cat = sample_line[categoria]
            label = sample_line[-6]
            axis = list(float(value) for value in sample_line[-20:])
            sample_info[label] = [cat, axis]
    return sample_info

# Representación gráfica con color

def represent_plot(coordenadas, labels):
    """
        A partir de un array de coordenadas, las representa en un gráfico.
        Labels indica a qué grupo pertenece y pintará todo el grupo del mismo color

        input:
        ---------------
        coordenadas: array con x puntos n-dimensionales
        label: array que indica a qué grupo pertenece cada punto
    """
    datos_3D = [[i[0], i[1], i[2]] for i in coordenadas]

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    colors = [plt.cm.Spectral(each) for each in np.linspace(0, 1, len(set(labels)))]

    for idx, point in enumerate(datos_3D):
        coordenates = point
        label = labels[idx]
        color = colors[label]
        if label == -1:
            color = 'black'
        ax.scatter(coordenates[0], coordenates[1], coordenates[2], c = color)
    plt.show()

def show_clustering_parameters(points, labels, name="Unknown"):
    Calinski_parameter = metrics.calinski_harabaz_score(points, labels)
    Silhouette_parameter = metrics.silhouette_score(points, labels)
    Davies_parameter = metrics.davies_bouldin_score(points, labels)
    number_of_labels = len(set(labels))
    print("%s\t%0.3f\t%0.3f\t%0.3f\t%i" % (name, Silhouette_parameter, Calinski_parameter, Davies_parameter, number_of_labels))

def contar(cl_1, cl_2):
    counts = Counter()
    for idx in range(len(cl_1)):
            counts[(cl_1[idx], cl_2[idx])] += 1
    return counts

def parametros(coordenadas, Manual_label, Cluster_label):
    print("Clasificación\tShilouette\tCalinski\tDavies\tNº Clusters")
    show_clustering_parameters(coordenadas, Manual_label, "Jose")
    show_clustering_parameters(coordenadas, Cluster_label, "Auto")

def barrido(EPS, min=8):
    curly_info = read_curly(TRADITOM, 13)
    sorted_curly_coordenates = list(curly_info[label][1] for  label in sorted(curly_info))
    Manual_labels = list(curly_info[label][0] for  label in sorted(curly_info))#Coordenadas
    for i in range(1, 31):
        cluster_labels = dbscan_cluster.labels_
        coordenadas = sorted_curly_coordenates
        eps = EPS/10+ EPS*i/10
        print(eps)
        cluster = db_cluster(coordenadas, eps, 4)
        cluster_labels = cluster.labels_
        show_clustering_parameters(coordenadas, cluster_labels)
        main(13, eps)

def cluster_and_plot(curly, filename, clusters):
    sorted_curly_coordenates = list(curly[label][1] for  label in sorted(curly))
    axis = len(sorted_curly_coordenates[0])
    Manual_labels = list(curly[label][0] for  label in sorted(curly))#Coordenadas
    # Clustering
    dbscan_cluster = cluster(sorted_curly_coordenates, clusters).fit(sorted_curly_coordenates)
    cluster_labels = dbscan_cluster.labels_
    coordenadas = sorted_curly_coordenates
    print("===============Representación gráfica=================\n")
    represent_plot(sorted_curly_coordenates, cluster_labels)
    conteo = contar(Manual_labels, cluster_labels)
    print("===============Comparación de clasificaciones===============\n")
    plott.plot_comparison_table(conteo, filename)
    print("===============Parámetros de clustering===============\n")
    show_clustering_parameters(sorted_curly_coordenates, cluster_labels, filename)
    write_curly(curly,cluster_labels, axis=axis)
    return dbscan_cluster

def cluster(data, clusters, linkage="ward" ):
    dbscan = AgglomerativeClustering(n_clusters=clusters, linkage=linkage)
    dbscan.fit(data)
    return dbscan

def write_curly(curly_info, labels, axis=5,filename="clustering_results.curly"):
    with open(filename, "w") as fhand:
        fhand.write("categories:rank1\tcategories:cluster\tlabel")
        for i in range(axis):
            fhand.write("\taxis{}".format(i+1))
        fhand.write("\n")
        for idx, key in enumerate(sorted(curly_info)):
            cat = curly_info[key][0]
            fhand.write(cat)
            fhand.write("\t"+str(labels[idx]))
            fhand.write("\t"+key)
            for axis in curly_info[key][1]:
                fhand.write("\t"+str(axis))
            fhand.write("\n")
        pass

def plot_comparison_table(table_counts, plot_path=None):

    size = 100

    y_pops = set()
    x_pops = set()
    for y_pop, x_pop in table_counts.keys():
        y_pops.add(y_pop)
        x_pops.add(x_pop)
    y_pops = sorted(y_pops)
    x_pops = sorted(x_pops)

    max_count = max(table_counts.values())

    counts = []
    x_poss = []
    y_poss = []
    for x_pos, x_pop in enumerate(x_pops):
        for y_pos, y_pop in enumerate(y_pops):
            count = table_counts.get((y_pop, x_pop), 0)
            if not count:
                continue
            count /= max_count
            x_poss.append(x_pos)
            y_poss.append(y_pos)
            counts.append(count)

    counts = numpy.array(counts) * size
    plt.figure()
    axes = plt.subplot(111)
    axes.scatter(x_poss, y_poss, s=counts, marker='s')

    axes.set_xticklabels(x_pops, horizontalalignment='right')
    axes.set_xticks(list(range(len(x_pops))))

    axes.set_yticklabels(y_pops)
    axes.set_yticks(list(range(len(y_pops))))
    if plot_path:
        plt.savefig(str(plot_path))
    plt.show()
    plt.close()
